body {
    margin: 0;
}

menu {
    margin: 0;
    padding: 0;
    display: inline;
}

.header {
    display: table;
    width: 100%;
    min-width: 10em;
}

.header h1 {
    display: table;
    margin: 0.5em 0 0em 0.5em;
    font-size: 200%;
    color: #000;
}
.header small {
    display: table;
    margin: 0 0 1em 1em;
    font-size: 85%;
}
.header a:link,
.header a:visited,
.header a:active {
    color: #fff;
    text-decoration: none;
}
.header a:hover {
   border-bottom: 1px dotted #fff; 
}

.pannel {
    clear: both;
    padding: 0.5em 1em;
    background-color: #B12929;
    background-image: linear-gradient(to bottom, #E66B6B, #B12929);
    background-repeat: repeat-x;
    border-radius: 3px 3px 3px;
}
.pannel menu.main {
}
.pannel menu.languages {
    display: table;
    float: right;
    text-align: right;
    font-size: 100%;
}
.pannel menu > .item {
    padding: 0.1em 0.3em 0.1em 0.3em;
}
.pannel menu > .selected > a {
    font-weight: bold;
}

.sidebar {
    float: left;
    width: 16em;
    padding: 1em 0 0 1em;
}
.sidebar a {
}
.sidebar h1 {
    margin: 0;
    font-size: 120%;
    font-weight: normal;
}
.sidebar h2 {
    margin: 0 0 0 1em;
    font-size: 100%;
    font-weight: normal;
}
.sidebar h3 {
    margin: 0 0 0 2em;
    font-size: 90%;
    font-weight: normal;
}

.linebar {
    float: left;
    margin: 0.5em 0 1.5em 0;
    padding: 0.5em;
    font-size: 11pt;
    background-color: #ddd;
    border: 1px dotted gray;
    border-radius: 6px 6px 6px;
    border-right: none;
}
.linebar h3 {
    margin: 0;
    font-size: 90%;
    font-weight: normal;
    text-align: right;
}

a:link,
a:visited,
a:hover,
a:active {
    color: #293CB1;
}

.page {
    clear: both;
    max-width: 62em;
    margin: 0 auto;
    padding: 0 1em 0 1em;
}

.content {
    margin-left: 17em;
    min-width: 16em;
}
.content >:first-child {
    margin-top: 0.5em;
}

@media (max-width: 50em){
    .content { margin: 0;}
    .sidebar { float: none;}
}

.footer {
    clear: both;
    padding-top: 2em;
    color: #333;
    font-size: 80%;
    text-align: center;
}
.footer .credit {
    padding: 0.5em;
}
.line {
    border-top: 1px solid #888;
}
.links {
    width: 5em;
    margin-right: 1em;
    float: right;
    text-align: right;
    font-weight: normal;
    font-family: sans-serif;
    font-size: 11pt;
}

h1, h2 {
    color: #293CB1;
}
h2 > i {
    font-size: 70%;
}
h2 > .links {
    line-height: 2em;
}
h2.module {
    margin: 2em 0 0 0;
    padding: 0.1em 0em 0.1em 0.5em;
    background-color: #B12929;
    background-image: linear-gradient(to bottom, #E66B6B, #B12929);
    border-radius: 3px 3px 3px;
    color: #222;
}
h2.module > .links > a {
    color: #bbe;
}

h3 > i {
    font-size: 60%;
    font-family: sans-serif;
    color: #000;
}
h3 > .links {
    line-height: 2em;
}
h3.function {
    margin: 2em 0 0 8em;
    text-indent: -8em;
    padding-bottom: 0;
    color: #293CB1;
    font-family: monospace;
    font-size: 170%;
}
h3.function > i {
    font-size: 45%;
}
h3.class {
    padding: 0.1em 0em 0.1em 0.5em;
    background-color: #ddd;
    border-radius: 3px 3px 3px;
    color: #293CB1;
    font-size: 130%;
}
h3.variable {
    font-family: monospace;
    font-size: 130%;
}

h4.method, h4.staticmethod, h4.descriptor {
    margin: 2em 0 0 8em;
    text-indent: -8em;
    padding-left: 2em;
    font-family: monospace;
    font-size: 130%;
}
h4.method > i, h4.staticmethod > i, h4.descriptor > i {
    font-size: 90%;
    color: #333
}
h4.staticmethod > i {
    margin-left: -9em;
}

h4.property {
    margin: 2em 0 0 0;
    padding-left: 2em;
    font-family: monospace;
    font-size: 120%;
}

h4.property > small, h4.staticmethod > small {
    color: #888;
}

div.module, div.class, div.method, div.staticmethod, div.descriptor, div.function, div.property {
    text-align: jus-tify;
    color: #333;
    margin: 0.7em 0 0 0;
    padding: 0 0 0 1em;
}
div.module > h3 {
    color: #000;
    font-size: 120%;
    margin: 0 0 0 -0.8em;
}
div.module > h4 {
    color: #000;
    font-size: 100%;
    margin: 0 0 0 -1em;
}
div.method, div.staticmethod, div.descriptor, div.property {
    padding-left: 3em;
}
pre {
    font-size: 10pt;
    margin: 0.5em 0 1.5em 0;
    padding: 0.5em;
    background-color: #ddd;
    border: 1px dotted gray;
    border-radius: 6px 6px 6px;
    overflow: auto;
}
pre > b {
    color: #b02c29;
}
pre > u, pre > var {
    color: #306900;
    text-decoration: none;
}
pre > i {
    color: #005089;
    font-style: normal;
}
pre > i > b {
    color: #CA8400;
    font-weight: normal;
}
pre > em, pre > kbd {
    color: #0000B0;
    font-style: normal;
}
pre > tt {
    color: #666;
}

.code, .doctest {
    padding-left: 0.5em;
    padding-right: 0.5em;
}

code {
    font-size: 11pt;
    color: #000;
}
code.param {
    white-space: pre;
    font-weight: bold;
    margin-left: 1em;
}
.submodules {
    padding: 0.2em 0.5em 0.2em 0.5em;
    margin: 0.5em 0 0.5em 0;
    background-color: #ddd;
    border-radius: 3px 3px 3px;
}
.submodules > .submodule {
    padding: 0 0 0 0.3em;
}


dt {
    font-weight: bold;
}
th, td {
    vertical-align: top;
}
table {
    margin-top: 1em;
}
td *, dd * {
    margin-top: 0.2em;
    margin-bottom: 0.4em;
}
.citations td:first-child {
    white-space: nowrap;
}
dd > ul {
    padding-left: 0;
}
